@@ -24,7 +24,7 @@ generate_pattern <- function(seqc, ordered_index=c(), append_to=list()) {
 #' \code{calculate_percent} is used to calculate dissimilarity index,
 #' proportion of isolates not in goi that have been discriminated against.
 #' 1 being all and 0 being none.
-#' @param pattern list of sequences
+#' @param pattern list of sequences' pattern (profile)
 #' @param goi group of interest
 #' @return Will return the dissimilarity index of the list of patterns.
 #' @export
@@ -39,9 +39,9 @@ calculate_percent <- function(pattern, goi) {
             target_seqs <- c(target_seqs, pattern[[isolate]])
         }
     }
-    isolate_w_goi_pattern <- length(which(pattern %in% target_seqs))
-    failed_to_discriminate <- isolate_w_goi_pattern - length(goi)
-    result <- 1 - (failed_to_discriminate / (length(pattern) - length(goi)))
+    isolate_w_goi_pattern <- as.numeric(length(which(pattern %in% target_seqs)))
+    failed_to_discriminate <- isolate_w_goi_pattern - as.numeric(length(goi))
+    result <- 1 - (failed_to_discriminate / (as.numeric(length(pattern)) - as.numeric(length(goi))))
     return(list(result = result))
 }
 
@@ -60,14 +60,14 @@ calculate_simpson <- function(pattern) {
 
     for (seqs in unique_sequences) {
         no_isolate_in_group[[seqs]] <-
-            length(which(pattern == seqs))
+            as.numeric(length(which(pattern == seqs)))
     }
 
     d_numerator <- Reduce(f = function(prevs, current) {
         return(prevs + (current * (current - 1)))
     }, x = no_isolate_in_group, init = 0)
 
-    d_denominator <- length(pattern) *
+    d_denominator <- as.numeric(length(pattern)) *
         (length(pattern) - 1)
 
     if (d_numerator == 0 || d_denominator == 0) {
@@ -389,7 +389,7 @@ branch_and_search <- function(starting_positions = c(),
 #' @param position position to check
 #' @param sequences sequences from group of interest
 #' @return return `TRUE` if the position contains multistate otherwise `FALSE`
-#' @keywords internal
+#' @export
 check_multistate <- function(position, sequences) {
     pattern <- generate_pattern(sequences, c(position))
     if (length(unique(pattern)) > 1) {
@@ -407,7 +407,6 @@ check_multistate <- function(position, sequences) {
 #' @inheritParams check_multistate
 #' @return return the value at that position,
 #' as well as base pattern for next iteration.
-#' @keywords internal
 #' @export
 cal_met_snp <- function(position, metric, seqc, prepend_position = c(), ...) {
     additional_args <- list(...)[[1]]
@@ -422,9 +421,12 @@ cal_met_snp <- function(position, metric, seqc, prepend_position = c(), ...) {
     }
     metric_function <- match.fun(metric)
     metric_expected_args <- formals(metric_function)
+
+    metric_expected_args_no_default <- names(metric_expected_args[sapply(metric_expected_args, is.symbol)])
+
     all_expected_args <- names(additional_args)[
         names(additional_args) %in% names(metric_expected_args)]
-    if (!all(names(metric_expected_args)
+    if (!all(names(metric_expected_args_no_default)
         [!names(metric_expected_args) == "pattern"] %in%
             all_expected_args)) {
 
@@ -434,6 +436,7 @@ cal_met_snp <- function(position, metric, seqc, prepend_position = c(), ...) {
               (!names(metric_expected_args) == "pattern")], collapse = ", "))
     }
     sub_args <- additional_args[all_expected_args]
+    sub_args <- sub_args[!is.na(names(sub_args))]
     sub_args[["pattern"]] <- pattern
     res <- do.call(metric_function, args = sub_args)
     if ("get_addn" %in% names(additional_args)) {
@@ -457,6 +460,7 @@ cal_met_snp <- function(position, metric, seqc, prepend_position = c(), ...) {
 #' @param seqc_length the length to iterate through.
 #' @keywords internal
 #' @importFrom BiocParallel SerialParam bplapply
+#' @importFrom utils tail
 #' @return Will return the resolution-optimised SNPs set, based on the metric.
 select_n_set_i_depth <- function(starting_positions = c(),
     excluded_positions = c(), seqc, metric,
@@ -464,7 +468,6 @@ select_n_set_i_depth <- function(starting_positions = c(),
     bp = SerialParam(), ...) {
 
     additional_args <- list(...)[[1]]
-    
     traversed <- list()
     #existing_pattern <- list()
     output_progress <- ifelse(
@@ -474,6 +477,14 @@ select_n_set_i_depth <- function(starting_positions = c(),
     # Calculate the score for the starting positions and select N best
     positions <- seq_len(seqc_length)
     positions <- as.list(positions[! positions %in% c(excluded_positions, starting_positions)])
+    if (number_of_result > length(positions)){
+        warning("Number of result is larger than the number of positions available.")
+        number_of_result <- length(positions)
+    }
+    if (length(positions) == 0) {
+        multi_result[[paste("result", 1)]] <- 
+            list("-" = 0)
+    }
     scores <- bplapply(positions,
         cal_met_snp, metric = metric, seqc = seqc,
         prepend_position = starting_positions, additional_args, 
@@ -506,17 +517,19 @@ select_n_set_i_depth <- function(starting_positions = c(),
     }
     if (output_progress) {
         first_pos <- paste(sapply(positions[position_order[1:number_of_result]], `[[`, 1), collapse = ", ")
-        cat("First SNPs for results are: ", first_pos, "\n")
+        cat("First SNPs for result(s) are: ", first_pos, "\n")
     }
     for (n in seq_len(number_of_result)) {
         current_level <- length(starting_positions) + 1
         current_selected_positions <- c()
         while (((current_level - length(starting_positions)) < max_depth) &&
-               (depth_1[position_order][[n]] < 1)) {
-            
+               (tail(result_d1[[n]], n = 1)[[1]] < 1)) {
             ###***
             positions <- seq_len(seqc_length)
-            positions <- as.list(positions[! positions %in% c(excluded_positions, selected_positions[1:n])])
+            positions <- as.list(positions[! positions %in% c(excluded_positions, selected_positions[1:n], current_selected_positions)])
+            if (length(positions) == 0) {
+                break
+            }
             scores <- bplapply(positions,
                 cal_met_snp, metric = metric, seqc = seqc,
                 prepend_position = traversed[[n]], additional_args,
@@ -527,7 +540,7 @@ select_n_set_i_depth <- function(starting_positions = c(),
             snps_1 <- bplapply(scores, function(score) {
                     return(score[["positions"]])
             }, BPPARAM = bp)
-
+            
             # Sorting for selection at this depth
             names(depth_1) <- snps_1
             position_order <- order(unlist(depth_1), decreasing = TRUE)
@@ -537,7 +550,6 @@ select_n_set_i_depth <- function(starting_positions = c(),
             result_d1[[n]][[
                     paste(traversed[[n]], collapse = ", ")]] <-
                         depth_1[position_order][[1]]
-
             ###***
             current_level = current_level + 1
             current_selected_positions <- c(current_selected_positions, snps_1[position_order[1]][[1]])
@@ -547,7 +559,7 @@ select_n_set_i_depth <- function(starting_positions = c(),
             result_d1[[n]]
         if (output_progress) {
             cat("Generated", n, "result\n")
-            cat("Selected SNPs are: ", traversed[[n]], "\n")
+            cat("Selected SNPs are: ", paste(traversed[[n]], collapse = ","), "\n", sep = "")
         }
     }
     return(multi_result)
@@ -570,26 +582,31 @@ select_n_set_i_depth <- function(starting_positions = c(),
 #' 0 means it will only calculate the metric for included position
 #' @param included_positions included positions
 #' @param excluded_positions excluded positions
+#' @param search_from search only from these positions, i.e.,
+#' any positions not in here are excluded, default to NULL
 #' @param iterate_included whether to calculate index
 #' at each level of the included SNPs
+#' @param completely_unique whether to identify completely unique SNPs set,
+#' default to FALSE, only the 1st SNP must be different
 #' @param ... other parameters as needed
 #' @importFrom BiocParallel SerialParam bplapply
+#' @importFrom utils tail
 #' @return Will return the resolution-optimised SNPs set, based on the metric.
 #' @export
 find_optimised_snps <- function(seqc, metric = "simpson", goi = c(),
     accept_multiallelic = TRUE, number_of_result = 1, max_depth = 1,
-    included_positions = c(), excluded_positions = c(),
-    iterate_included = FALSE, bp = SerialParam(), ...) {
+    included_positions = c(), excluded_positions = c(), search_from = NULL,
+    iterate_included = FALSE, completely_unique = FALSE, bp = SerialParam(), ...) {
 
     # Define parameters
     if (inherits(seqc, "processed_seqs")) {
         if (seqc$check_length){
             all_length <- seqc$length
         } else {
-            all_length <- unlist(bplapply(seqc[["seqc"]], length, BPPARAM = bp))
+            all_length <- lengths(seqc[["seqc"]])
         }
     } else {
-        all_length <- unlist(bplapply(seqc, length, BPPARAM = bp))
+        all_length <- lengths(seqc)
     }
 
     if (length(unique(all_length)) > 1) {
@@ -623,6 +640,12 @@ find_optimised_snps <- function(seqc, metric = "simpson", goi = c(),
             additional_exclude]
     }
 
+    if (!is.null(search_from)) {
+        additional_exclude <- positions[!positions %in% search_from]
+        positions <- positions[! positions %in%
+            additional_exclude]
+    }
+
     # Check if any of the included positions are in excluded positions
     if (any(included_positions %in% excluded_positions)) {
         errors <- included_positions[included_positions %in% excluded_positions]
@@ -633,12 +656,11 @@ find_optimised_snps <- function(seqc, metric = "simpson", goi = c(),
     # Check if all required parameters are provided
     all_parameters <- list(...)
     all_parameters[["goi"]] <- goi
-
     metric_fun <- get_metric_fun(metric)[["calc"]]
     check_args <- get_metric_fun(metric)[["args"]]
     if (! is.null(check_args)) {
         if (! check_args(all_parameters)) {
-            stop("GOI must be defined in percent mode")
+            stop("Not all required arguments are supplied for the selected mode")
         }
     }
 
@@ -679,10 +701,27 @@ find_optimised_snps <- function(seqc, metric = "simpson", goi = c(),
         ) {
         all_result <- list(result = result)
     } else {
-        branch_result <- select_n_set_i_depth(included_positions,
-            c(additional_exclude, excluded_positions),  sequences, metric_fun,
-            number_of_result, max_depth, min(all_length),
-            bp, all_parameters)
+        if (completely_unique) {
+            new_excluded <- c()
+            branch_result <- list()
+            for (n_res in seq_len(number_of_result)) {
+                temp_result <- select_n_set_i_depth(included_positions,
+                    c(new_excluded, additional_exclude, excluded_positions),  sequences, metric_fun,
+                    1, max_depth, min(all_length),
+                    bp, all_parameters)
+                SSNPS <- suppressWarnings(as.numeric(tail(strsplit(names(temp_result[[1]]), split = ", "), n = 1)[[1]]))
+                SSNPS <- SSNPS[!is.na(SSNPS)]
+                new_excluded <- c(new_excluded, SSNPS)
+                branch_result[[n_res]] <- temp_result[[1]]
+            }
+            names(branch_result) <- paste("Result", seq_len(number_of_result))
+        } else {
+            branch_result <- select_n_set_i_depth(included_positions,
+                c(additional_exclude, excluded_positions),  sequences, metric_fun,
+                number_of_result, max_depth, min(all_length),
+                bp, all_parameters)
+        }
+        
         all_result <- bplapply(branch_result, function(branch_r, inc_r) {
             return(c(inc_r, branch_r))
         }, inc_r = result, BPPARAM = SerialParam())
@@ -695,4 +734,77 @@ find_optimised_snps <- function(seqc, metric = "simpson", goi = c(),
         seqc_name = deparse(substitute(seqc)), goi = goi,
         all_sequences = names(sequences),
         max_depth = max_depth, metric = metric))
+}
+
+
+
+#' \code{identify_group_variant_breakdown}
+#'
+#' @description
+#' \code{calculate_variant_within_group} is used to identify proportion of different samples
+#' having the same profile.
+#' @param pattern list of sequences' pattern (profile)
+#' @param meta metadata of the sequences
+#' @param target column name of the target group
+#' @param get_count whether to return the count of samples rather than the raw number, default to FALSE.
+#' @return Will return the Simpson's index of the list of patterns.
+#' @importFrom data.table dcast .N
+#' @export
+calculate_variant_within_group <- function(pattern, meta, target, get_count = FALSE) {
+    data.table::setDT(meta)
+    . <- NULL
+    colnames(meta)[colnames(meta) == target] <- "target"
+    npattern <- data.table(pattern = unname(unlist(pattern)),
+        target = meta[match(names(pattern), meta$isolate), ]$target)
+    cubed <- data.table::cube(npattern, .(count = .N), by = c("pattern", "target"))
+    cubed <- cubed[!is.na(cubed$pattern) & !is.na(cubed$target), ]
+    res <- dcast(cubed, cubed$pattern ~ cubed$target, value.var = "count")
+    res[is.na(res)] <- 0
+    colnames(res)[1] <- "allele"
+    if (get_count) {
+        return(list(result = res))
+    }
+    alleles <- res[,1]
+    columns <- colnames(res)[-1]
+    res <- do.call(cbind, lapply(res[,columns, with = FALSE], function(col) {
+        sapply(col, function(cell){
+            return(cell/sum(col))
+        })
+    }))
+
+    return(list(result = cbind(alleles, res)))
+}
+
+#' \code{iterate_through}
+#'
+#' @description
+#' \code{iterate_through} is used to calculate the metric at each position
+#' @inheritParams find_optimised_snps
+#' @inheritParams check_multistate
+#' @importFrom BiocParallel MulticoreParam bplapply
+#' @return return a dataframe containing the position and result.
+#' @export
+iterate_through <- function(metric, seqc, bp = MulticoreParam(), ...){
+    if (inherits(seqc, "processed_seqs")) {
+        if (seqc$check_length){
+            all_length <- seqc$length
+        } else {
+            all_length <- lengths(seqc[["seqc"]])
+        }
+    } else {
+        all_length <- lengths(seqc)
+    }
+    if (length(unique(all_length)) > 1) {
+        warning("Sequences are not of the same length; ",
+        "Only the first ", min(all_length), " positions will be used")
+    }
+    positions <- seq_len(min(all_length))
+    scores <- bplapply(positions, cal_met_snp, metric = metric, seqc = seqc,
+        prepend_position = c(), list(...),
+            BPPARAM = bp)
+    rr <- bplapply(scores, function(x){
+        res <- x$result
+        return(cbind(position = x$position, res))
+    }, BPPARAM = bp)
+    return(rbindlist(rr))
 }
\ No newline at end of file
